@@ -343,7 +343,7 @@ gnome_exit (void)
 }
 
 static void
-do_desktop_rescan_devices (void)
+do_rescan_desktop (void)
 {
 	desktop_reload_icons (FALSE, 0, 0);
 }
@@ -357,10 +357,15 @@ gnome_launch_mime_editor (void)
 void configure_box (void);
 
 GtkCheckMenuItem *gnome_toggle_snap (void);
+
 static GnomeUIInfo gnome_panel_new_menu [] = {
-	 GNOMEUIINFO_ITEM_NONE(N_("_Terminal"), N_("Launch a new terminal in the current directory"), gnome_open_terminal),
+	GNOMEUIINFO_ITEM_NONE(N_("_Terminal"),
+			      N_("Launch a new terminal in the current directory"), gnome_open_terminal),
 	/* If this ever changes, make sure you update create_new_menu accordingly. */
-	GNOMEUIINFO_ITEM_NONE(N_("_Directory..."), N_("Creates a new directory"), gnome_mkdir_cmd),
+	GNOMEUIINFO_ITEM_NONE(N_("_Directory..."),
+			      N_("Creates a new directory"), gnome_mkdir_cmd),
+	GNOMEUIINFO_ITEM_NONE(N_("_File..."),
+			      N_("Creates a new file in this directory"), gnome_newfile_cmd),
 	GNOMEUIINFO_END
 };
 
@@ -387,7 +392,9 @@ GnomeUIInfo gnome_panel_file_menu [] = {
 	GNOMEUIINFO_SEPARATOR,
 	GNOMEUIINFO_ITEM_NONE(N_("Show directory sizes"), N_("Shows the disk space used by each directory"), dirsizes_cmd),
 	GNOMEUIINFO_SEPARATOR,
-	GNOMEUIINFO_MENU_CLOSE_WINDOW_ITEM(gnome_close_panel, NULL),
+	{ GNOME_APP_UI_ITEM, N_("Close window"), N_("Closes this window"),
+	  gnome_close_panel, NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_CLOSE,
+	  'w', GDK_CONTROL_MASK, NULL },
         GNOMEUIINFO_END
 };
 
@@ -454,8 +461,9 @@ GnomeUIInfo gnome_panel_about_menu [] = {
 GnomeUIInfo gnome_panel_desktop_menu [] = {
 	GNOMEUIINFO_SUBTREE(N_("_Arrange Icons"), desktop_arrange_icons_items),
 	GNOMEUIINFO_SEPARATOR,
-	GNOMEUIINFO_ITEM_NONE (N_("Recreate Desktop _Shortcuts"), NULL, desktop_rescan_devices),
-	GNOMEUIINFO_ITEM_NONE (N_("Rescan _Desktop"), NULL, do_desktop_rescan_devices),
+	GNOMEUIINFO_ITEM_NONE (N_("Rescan _Desktop Directory"), NULL, do_rescan_desktop),
+	GNOMEUIINFO_ITEM_NONE (N_("Rescan De_vices"), NULL, desktop_rescan_devices),
+	GNOMEUIINFO_ITEM_NONE (N_("Recreate Default _Icons"), NULL, desktop_recreate_default_icons),
 	GNOMEUIINFO_END
 };
 
@@ -715,7 +723,7 @@ copy_uiinfo_widgets (GnomeUIInfo *uiinfo)
 }
 
 WPanel *
-create_container (Dlg_head *h, char *name, char *geometry)
+create_container (Dlg_head *h, const char *name, const char *geometry)
 {
 	PanelContainer *container;
 	WPanel *panel;
@@ -804,7 +812,7 @@ new_panel_with_geometry_at (const char *dir, const char *geometry)
 {
 	WPanel *panel;
 
-	mc_chdir (dir);
+	mc_chdir ((char *) dir);
 	panel = create_container (desktop_dlg, dir, geometry);
 	set_current_panel (panel);
 	add_widget (desktop_dlg, panel);
