@@ -30,7 +30,7 @@
 #include "tree.h"
 #include "../vfs/vfs.h"
 
-/* "$Id: dir.c,v 1.20 1999/11/03 12:50:54 pavel Exp $" */
+/* "$Id: dir.c,v 1.21 2000/04/14 08:58:37 timur Exp $" */
 
 /*
  * If true show files starting with a dot.
@@ -76,7 +76,7 @@ sort_orders_t sort_orders [SORT_TYPES_TOTAL] = {
     { N_("&Group"),       sort_group }
 };
 
-#define string_sortcomp(a,b) (case_sensitive ? strcmp (a,b) : g_strcasecmp (a,b))
+#define string_sortcomp(a,b) (case_sensitive ? strcoll (a,b) : g_strcasecmp (a,b))
 
 int
 unsorted (const file_entry *a, const file_entry *b)
