@@ -3,7 +3,7 @@
  *
  * Written at 1998 by Pavel Machek <pavel@ucw.cz>, distribute under LGPL.
  *
- * $Id: direntry.c,v 1.27 2000/01/31 13:48:00 pavel Exp $
+ * $Id: direntry.c,v 1.32 2000/05/03 19:10:52 pavel Exp $
  *
  * Very loosely based on tar.c from midnight and archives.[ch] from
  * avfs by Miklos Szeredi (mszeredi@inf.bme.hu)
@@ -39,18 +39,13 @@ vfs_s_new_inode (vfs *me, vfs_s_super *super, struct stat *initstat)
 {
     vfs_s_inode *ino;
 
-    ino = g_new (vfs_s_inode, 1);
+    ino = g_new0 (vfs_s_inode, 1);
     if (!ino)
 	return NULL;
 
-    ino->linkname = ino->localname = NULL;
-    ino->subdir = NULL;
     if (initstat)
         ino->st = *initstat;
     ino->super = super;
-    ino->ent = NULL;
-    ino->flags = 0;
-    ino->st.st_nlink = 0;
     ino->st.st_ino = MEDATA->inode_counter++;
     ino->st.st_dev = MEDATA->rdev;
 
@@ -67,16 +62,12 @@ vfs_s_new_entry (vfs *me, char *name, vfs_s_inode *inode)
 {
     vfs_s_entry *entry;
 
-    entry = g_new (struct vfs_s_entry, 1);
+    entry = g_new0 (struct vfs_s_entry, 1);
     total_entries++;
 
     if (name)
 	entry->name = g_strdup (name);
-    else
-	entry->name = NULL;
-    entry->dir = NULL;
-    entry->next = NULL;
-    entry->prevp = NULL;
+
     entry->ino = inode;
     entry->ino->ent = entry;
     CALL (init_entry) (me, entry);
@@ -229,17 +220,16 @@ vfs_s_find_entry_tree (vfs *me, vfs_s_inode *root, char *path, int follow, int f
 {
     unsigned int pseg;
     vfs_s_entry *ent = NULL;
-    int found;
     char p[MC_MAXPATHLEN] = "";
 
-    while (1){
+    while (root){
 	int t;
 
-	for (pseg = 0; path [pseg] == PATH_SEP; pseg++)
-		;
-	if (!path [pseg])
+	while (*path == PATH_SEP)	/* Strip leading '/' */
+	    path++;
+
+	if (!path [0])
 	    return ent;
-	path += pseg;
 
 	for (pseg = 0; path[pseg] && path[pseg] != PATH_SEP; pseg++)
 		;
@@ -248,7 +238,6 @@ vfs_s_find_entry_tree (vfs *me, vfs_s_inode *root, char *path, int follow, int f
 	strncpy (p + (t = strlen (p)), path, pseg);
 	p[t + pseg] = '\0';
 
-	found = 0;
 	for (ent = root->subdir; ent != NULL; ent = ent->next)
 	    if (strlen (ent->name) == pseg && (!strncmp (ent->name, path, pseg)))
 		/* FOUND! */
@@ -263,6 +252,8 @@ vfs_s_find_entry_tree (vfs *me, vfs_s_inode *root, char *path, int follow, int f
 	    return NULL;
 	root = ent->ino;
     }
+
+    return NULL;
 }
 
 static void
@@ -278,10 +269,7 @@ split_dir_name (vfs *me, char *path, char **dir, char **name, char **save)
 	*save = s;
 	*dir = path;
 	*s++ = 0;
-	if (!*s)	/* This can happen if someone does stat("/"); */
-	    *name = "";
-	else
-	    *name = s;
+	*name = s;
     }
 }
 
@@ -376,7 +364,9 @@ vfs_s_resolve_symlink (vfs *me, vfs_s_entry *entry, char *path, int follow)
         if (*linkname == PATH_SEP)
 	    return (MEDATA->find_entry) (me, entry->dir->super->root, linkname, follow - 1, 0);
 	else { /* FIXME: this does not work */ 
-	    sprintf(buf, "%s/%s", vfs_s_fullpath(me, entry->dir), linkname);
+	    char *fullpath = vfs_s_fullpath(me, entry->dir);
+	    sprintf(buf, "%s/%s", fullpath, linkname);
+	    g_free (fullpath);
 	    return (MEDATA->find_entry) (me, entry->dir->super->root, buf, follow - 1, 0);
 	}
     }
@@ -411,10 +401,6 @@ vfs_s_new_super (vfs *me)
     vfs_s_super *super;
 
     super = g_new0 (struct vfs_s_super, 1);
-    super->root = NULL;
-    super->name = NULL;
-    super->fd_usage = 0;
-    super->ino_usage = 0;
     super->me = me;
     return super;
 }
@@ -486,7 +472,7 @@ vfs_s_get_path_mangle (vfs *me, char *inname, struct vfs_s_super **archive, int
     char *local, *op, *archive_name;
     int result = -1;
     struct vfs_s_super *super;
-    void *cookie;
+    void *cookie = NULL;
     
     archive_name = inname;
     vfs_split (inname, &local, &op);
@@ -557,7 +543,7 @@ vfs_s_fullpath (vfs *me, vfs_s_inode *ino)
 	ERRNOR (EAGAIN, NULL);
 
     if ((!ino->ent->dir) || (!ino->ent->dir->ent)) /* It must be directory */
-        return g_strconcat( ino->ent->name, NULL );
+        return g_strdup (ino->ent->name);
 
     return  g_strconcat (ino->ent->dir->ent->name, PATH_SEP_STR, 
 			    ino->ent->name, NULL);
@@ -579,13 +565,17 @@ vfs_s_inode *
 vfs_s_inode_from_path (vfs *me, char *name, int flags)
 {
     struct vfs_s_super *super;
+    struct vfs_s_inode *ino;
     char *q;
 
     if (!(q = vfs_s_get_path_mangle (me, name, &super, 0))) 
 	return NULL;
 
-    return vfs_s_find_inode (me, super->root, q, flags & FL_FOLLOW ? LINK_FOLLOW : LINK_NO_FOLLOW, flags & ~FL_FOLLOW);
-
+    ino = vfs_s_find_inode (me, super->root, q, flags & FL_FOLLOW ? LINK_FOLLOW : LINK_NO_FOLLOW, flags & ~FL_FOLLOW);
+    if ((!ino) && (!*q))
+	/* We are asking about / directory of ftp server: assume it exists */
+	ino = vfs_s_find_inode (me, super->root, q, flags & FL_FOLLOW ? LINK_FOLLOW : LINK_NO_FOLLOW, FL_DIR | (flags & ~FL_FOLLOW));
+    return ino;
 }
 
 struct dirhandle {
@@ -701,7 +691,7 @@ vfs_s_internal_stat (vfs *me, char *path, struct stat *buf, int flag)
     struct vfs_s_inode *ino;
 
     if (!(ino = vfs_s_inode_from_path (me, path, flag)))
-	return -1;
+        return -1;
     *buf = ino->st;
     return 0;
 }
@@ -917,8 +907,10 @@ vfs_s_close (void *fh)
 	char *s = vfs_s_fullpath (me, FH->ino);
 	if (!s)
 	    res = -1;
-	else
+ 	else {
 	    res = MEDATA->file_store (me, FH_SUPER, s, FH->ino->localname);
+	    g_free (s);
+	}
 	vfs_s_invalidate (me, FH_SUPER);
     }
     if (FH->handle)
