@@ -5,7 +5,7 @@
                1995 Jakub Jelinek
 	       1998 Pavel Machek
    
-   $Id: vfs.c,v 1.54 2000/03/06 16:31:37 pavel Exp $
+   $Id: vfs.c,v 1.58 2000/05/03 19:10:52 pavel Exp $
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public License
@@ -234,6 +234,7 @@ vfs_split (char *path, char **inpath, char **op)
 	return ret;
     }
 
+
     if (slash)
 	*slash = PATH_SEP;
     ret = vfs_split (path, inpath, op);
@@ -248,11 +249,9 @@ vfs_rosplit (char *path)
     char *slash;
     vfs *ret;
 
-    if (!path)
-	vfs_die ("Can not rosplit NULL");
-    
+    g_return_val_if_fail(path, NULL);
+
     semi = strrchr (path, '#');
-    
     if (!semi || !path_magic (path))
 	return NULL;
     
@@ -262,6 +261,8 @@ vfs_rosplit (char *path)
 	*slash = 0;
     
     ret = vfs_type_from_op (semi+1);
+    if (!ret && (vfs_flags & FL_NO_LOCALHASH))
+	return &vfs_nil_ops;
 
     if (slash)
 	*slash = PATH_SEP;
@@ -280,7 +281,7 @@ vfs_type (char *path)
     vfs = vfs_rosplit(path);
 
     if (!vfs)
-        vfs = &vfs_local_ops;
+	vfs = &vfs_local_ops;
 
     return vfs;
 }
@@ -409,8 +410,10 @@ mc_open (const char *filename, int flags, ...)
     mode = va_arg (ap, int);
     va_end (ap);
     
-    if (!vfs->open)
-	vfs_die ("VFS must support open.\n");
+    if (!vfs->open) {
+	errno = -EOPNOTSUPP;
+	return -1;
+    }
 
     info = (*vfs->open) (vfs, file, flags, mode);	/* open must be supported */
     g_free (file);
@@ -1071,9 +1074,8 @@ mc_def_getlocalcopy (vfs *vfs, char *filename)
     fdout = open (tmp, O_CREAT|O_WRONLY|O_TRUNC|O_EXCL, 0600);
     if (fdout == -1)
 	goto fail;
-    while ((i = mc_read (fdin, buffer, sizeof (buffer))) == sizeof (buffer)){
+    while ((i = mc_read (fdin, buffer, sizeof (buffer))) > 0)
         write (fdout, buffer, i);
-    }
     if (i == -1)
 	goto fail;
     if (write (fdout, buffer, i)==-1)
@@ -1093,7 +1095,7 @@ mc_def_getlocalcopy (vfs *vfs, char *filename)
  fail:
     if (fdout) close(fdout);
     if (fdin) mc_close (fdin);
-    free (tmp);
+    g_free (tmp);
     return NULL;
 }
 
@@ -1790,7 +1792,7 @@ error:
 
       if (++errorcount < 5) {
 	message_1s (1, _("Could not parse:"), p_copy);
-      } else if (errorcount == 5)
+      } else if (errorcount >= 5)
 	message_1s (1, _("More parsing errors will be ignored."), _("(sorry)"));
     }
 
